"""Somesy implementation for Java Maven pom.xml."""

# some POM-related constants and reusable objects
POM_URL = "http://maven.apache.org/POM/4.0.0"
POM_PREF = "{" + POM_URL + "}"
POM_ROOT_ATRS = {
    "xmlns": "http://maven.apache.org/POM/4.0.0",
    "xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
    "xsi:schemaLocation": "http://maven.apache.org/POM/4.0.0 http://maven.apache.org/xsd/maven-4.0.0.xsd",
}
